>From 96839d4e6c360fa02978d3d355af9de1b5400428 Mon Sep 17 00:00:00 2001
From: =?utf-8?q?Cyril=20Math=C3=A9?= <cmathe@actech-innovation.com>
Date: Fri, 10 Jul 2009 09:48:07 +0200
Subject: [PATCH] libvlc API: Add equalizer in audio.c and media_player.c

 Getters:
   libvlc_audio_equalizer_get_amp_at_index: get a band value
   libvlc_audio_equalizer_get_band_count: get the number of band
   libvlc_audio_equalizer_get_preamp: get preamp value
   libvlc_audio_equalizer_presets_get_count: get the number of equalizer presets
   libvlc_audio_equalizer_get_preset_name: get equalizer preset name

 Setters:
   libvlc_audio_equalizer_set_amp_at_index: set a new band value
   libvlc_audio_equalizer_set_preamp: set a new preamp value
   libvlc_audio_equalizer_set_preset: set a new equalizer preset

 Others:
   libvlc_audio_equalizer_new: create a new libvlc_audio_equalizer_t
   libvlc_audio_equalizer_release: release a libvlc_audio_equalizer_t

 Equalizer Setter and Getter
   libvlc_media_player_get_equalizer: get current equalizer settings
   libvlc_media_player_set_equalizer: set a new equalizer setting
---
 include/vlc/libvlc_media_player.h |  111 ++++++++++++++++++++
 src/control/audio.c               |  149 ++++++++++++++++++++++++++
 src/control/libvlc_internal.h     |    7 ++
 src/control/media_player.c        |  209 +++++++++++++++++++++++++++++++++++++
 src/libvlc.sym                    |   12 ++
 5 files changed, 488 insertions(+), 0 deletions(-)

diff --git a/include/vlc/libvlc_media_player.h b/include/vlc/libvlc_media_player.h
index d44c006..ba87a04 100644
--- a/include/vlc/libvlc_media_player.h
+++ b/include/vlc/libvlc_media_player.h
@@ -98,6 +98,10 @@ typedef enum libvlc_video_marquee_string_option_t {
     libvlc_marquee_Text = 0
 } libvlc_video_marquee_string_option_t;
 
+/**
+ * Equalizer Opaque Structure
+ */
+typedef struct libvlc_audio_equalizer_t libvlc_audio_equalizer_t;
 
 /**
  * Create an empty Media Player object
@@ -1034,6 +1038,113 @@ VLC_PUBLIC_API void libvlc_audio_set_channel( libvlc_instance_t *,
                                               int,
                                               libvlc_exception_t * );
 
+/**
+ * Get the number or equalizer presets
+ *
+ */
+VLC_PUBLIC_API unsigned libvlc_audio_equalizer_presets_get_count();
+
+/**
+ * Get a preset name
+ *
+ * \param i_index equalizer preset identifier
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API const char * libvlc_audio_equalizer_get_preset_name( int, libvlc_exception_t * );
+
+/**
+ * Create a new libvlc_audio_equalizer_t
+ *
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API libvlc_audio_equalizer_t * libvlc_audio_equalizer_new( libvlc_exception_t * );
+
+/**
+ * Release a libvlc_audio_equalizer_t
+ *
+ * \param p_eqz libvlc_audio_equalizer_t object to release
+ */
+VLC_PUBLIC_API void libvlc_audio_equalizer_release( libvlc_audio_equalizer_t * );
+
+/**
+ * Set a preset: set preset values into a libvlc_audio_equalizer_t
+ *
+ * \param p_eqz libvlc_audio_equalizer_t object
+ * \param i_index equalizer preset identifier
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_audio_equalizer_set_preset( libvlc_audio_equalizer_t *,
+                                                       int,
+                                                       libvlc_exception_t * );
+
+/**
+ * Get the number of band in current equalizer
+ *
+ * \param p_eqz libvlc_audio_equalizer_t object
+ */
+VLC_PUBLIC_API unsigned libvlc_audio_equalizer_get_band_count( libvlc_audio_equalizer_t * );
+
+/**
+ * Get a band value, the band to get is defined by i_bandID
+ *
+ * \param p_eqz libvlc_audio_equalizer_t object
+ * \param i_bandID band identifier
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API float libvlc_audio_equalizer_get_amp_at_index( libvlc_audio_equalizer_t *,
+                                                              int, libvlc_exception_t * );
+
+/**
+ * Get the preamp value
+ *
+ * \param p_eqz libvlc_audio_equalizer_t object
+ */
+VLC_PUBLIC_API float libvlc_audio_equalizer_get_preamp( libvlc_audio_equalizer_t * );
+
+/**
+ * Set a new band value, the band to set is defined by i_bandID
+ *
+ * \param p_eqz libvlc_audio_equalizer_t object
+ * \param i_bandID band identifier
+ * \param f_band band value to set
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_audio_equalizer_set_amp_at_index( libvlc_audio_equalizer_t *,
+                                                             int, float,
+                                                             libvlc_exception_t * );
+
+/**
+ * Sey a new preamp value
+ *
+ * \param p_eqz libvlc_audio_equalizer_t object
+ * \param f_preamp preamp value to set
+ */
+VLC_PUBLIC_API void libvlc_audio_equalizer_set_preamp( libvlc_audio_equalizer_t *,
+                                                       float);
+
+/**
+ * Get a libvlc_audio_equalizer_t object
+ *
+ * \param p_instance libvlc instance
+ * \param p_eqz libvlc_audio_equalizer_t object
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_media_player_get_equalizer( libvlc_instance_t *,
+                                                       libvlc_audio_equalizer_t *,
+                                                       libvlc_exception_t * );
+
+/**
+ * Set a libvlc_audio_equalizer_t object
+ *
+ * \param p_instance libvlc instance
+ * \param p_eqz libvlc_audio_equalizer_t object
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_media_player_set_equalizer( libvlc_instance_t *,
+                                                       libvlc_audio_equalizer_t *,
+                                                       libvlc_exception_t * );
+
+
 /** @} audio */
 
 /** @} media_player */
diff --git a/src/control/audio.c b/src/control/audio.c
index 0146bbe..b625165 100644
--- a/src/control/audio.c
+++ b/src/control/audio.c
@@ -36,6 +36,7 @@
 
 #include "libvlc_internal.h"
 #include "media_player_internal.h"
+#include "modules/audio_filter/equalizer_presets.h"
 
 /*
  * Remember to release the returned aout_instance_t since it is locked at
@@ -511,3 +512,151 @@ void libvlc_audio_set_channel( libvlc_instance_t *p_instance,
     }
 
 }
+
+/*****************************************************************************
+ * Equalizer: Getters and Setters on libvlc_audio_player_t opaque struct
+ *****************************************************************************/
+/*****************************************************************************
+ * libvlc_audio_equalizer_presets_get_count: return preset number
+ *****************************************************************************/
+
+unsigned libvlc_audio_equalizer_presets_get_count()
+{
+    return NB_PRESETS;
+}
+
+/*****************************************************************************
+ * libvlc_audio_equalizer_preset_name: return a preset name
+ *****************************************************************************/
+const char * libvlc_audio_equalizer_get_preset_name( int i_index,
+                                                 libvlc_exception_t *p_e)
+{
+    if( ( i_index < 0 ) || ( i_index > NB_PRESETS ) )
+    {
+        libvlc_exception_raise( p_e, "No preset equalizer corresponding" );
+        return;
+    }
+    return eqz_preset_10b[i_index]->psz_name;
+}
+
+/*****************************************************************************
+ * libvlc_audio_equalizer_new: create a libvlc_audio_equalizer_t
+ *****************************************************************************/
+libvlc_audio_equalizer_t *
+libvlc_audio_equalizer_new( libvlc_exception_t *p_e )
+{
+    libvlc_audio_equalizer_t * p_eqz;
+
+    p_eqz = malloc(sizeof(libvlc_audio_equalizer_t));
+
+    if( !p_eqz )
+        return NULL;
+
+    /* initialize equalizer values */
+    p_eqz->f_preamp = 12.0;
+    p_eqz->i_band = 10;
+    for( int i = 0 ; i < 10 ; i++ )
+        p_eqz->f_band[i] = 0.0;
+
+    return p_eqz;
+}
+
+/*****************************************************************************
+ * libvlc_audio_equalizer_release: release a libvlc_audio_equalizer_t
+ *****************************************************************************/
+void libvlc_audio_equalizer_release(libvlc_audio_equalizer_t * p_eqz)
+{
+    free( p_eqz );
+}
+
+/*****************************************************************************
+ * libvlc_audio_equalizer_set_preset: set a new preset, convert preset values
+ *                                    into libvlc_audio_equalizer_t
+ *****************************************************************************/
+void libvlc_audio_equalizer_set_preset( libvlc_audio_equalizer_t * p_eqz,
+                                        int i_preset_index,
+                                        libvlc_exception_t *p_e)
+{
+    if( ( i_preset_index < 0 ) || ( i_preset_index >= NB_PRESETS ) )
+    {
+        libvlc_exception_raise( p_e, "No preset equalizer corresponding" );
+        return;
+    }
+    p_eqz->i_band = eqz_preset_10b[i_preset_index]->i_band;
+    p_eqz->f_preamp = eqz_preset_10b[i_preset_index]->f_preamp;
+    for(int i = 0 ; i < EQZ_BANDS_MAX ; i++)
+        p_eqz->f_band[i] = eqz_preset_10b[i_preset_index]->f_amp[i];
+}
+
+/*****************************************************************************
+ * libvlc_audio_equalizer_get_band_count: retun the band number
+ *****************************************************************************/
+unsigned libvlc_audio_equalizer_get_band_count(libvlc_audio_equalizer_t * p_eqz)
+{
+    return p_eqz->i_band;
+}
+
+/*****************************************************************************
+ * libvlc_audio_equalizer_get_amp_at_index: return a band value
+ *****************************************************************************/
+float libvlc_audio_equalizer_get_amp_at_index( libvlc_audio_equalizer_t * p_eqz,
+                                               int i_bandID,
+                                               libvlc_exception_t * p_e)
+{
+    int i_band = libvlc_audio_equalizer_get_band_count( p_eqz );
+
+    if( ( i_bandID < 0 ) || ( i_bandID >= i_band ))
+    {
+        libvlc_exception_raise(p_e, "Invalid band number" );
+        return -1;
+    }
+
+    return p_eqz->f_band[i_bandID];
+}
+
+/*****************************************************************************
+ * libvlc_audio_equalizer_get_preamp: return preamp value
+ *****************************************************************************/
+float libvlc_audio_equalizer_get_preamp(libvlc_audio_equalizer_t * p_eqz)
+{
+    return p_eqz->f_preamp;
+}
+
+/*****************************************************************************
+ * libvlc_audio_equalizer_set_amp_at_index: set a new band value
+ *****************************************************************************/
+void libvlc_audio_equalizer_set_amp_at_index(libvlc_audio_equalizer_t * p_eqz,
+                                             int i_bandID, float f_band,
+                                            libvlc_exception_t * p_e )
+{
+    int i_band = libvlc_audio_equalizer_get_band_count( p_eqz );
+
+    if( ( i_bandID < 0 ) || ( i_bandID >= i_band ))
+    {
+        libvlc_exception_raise(p_e, "Invalid band number" );
+        return;
+    }
+
+    /* don't set inconsistent values */
+    if( f_band > 20)
+        f_band = 20;
+    if( f_band < -20)
+        f_band = -20;
+
+    p_eqz->f_band[i_bandID] = f_band;
+}
+
+/*****************************************************************************
+ * libvlc_audio_equalizer_set_preamp: set a new preamp value
+ *****************************************************************************/
+void libvlc_audio_equalizer_set_preamp(libvlc_audio_equalizer_t * p_eqz,
+                                       float f_preamp)
+{
+    /* don't set inconsistent values */
+    if( f_preamp > 20)
+        f_preamp = 20;
+    if( f_preamp < -20)
+        f_preamp = -20;
+
+    p_eqz->f_preamp = f_preamp;
+}
diff --git a/src/control/libvlc_internal.h b/src/control/libvlc_internal.h
index 64d8941..11ac6cd 100644
--- a/src/control/libvlc_internal.h
+++ b/src/control/libvlc_internal.h
@@ -24,6 +24,7 @@
 
 #ifndef _LIBVLC_INTERNAL_H
 #define _LIBVLC_INTERNAL_H 1
+#define EQZ_BANDS_MAX 10
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -78,6 +79,12 @@ struct libvlc_instance_t
     struct libvlc_callback_entry_list_t *p_callback_list;
 };
 
+struct libvlc_audio_equalizer_t
+{
+    float f_preamp;
+    float f_band[EQZ_BANDS_MAX];
+    unsigned int i_band;
+};
 
 /***************************************************************************
  * Other internal functions
diff --git a/src/control/media_player.c b/src/control/media_player.c
index b314bde..72dc474 100644
--- a/src/control/media_player.c
+++ b/src/control/media_player.c
@@ -34,6 +34,8 @@
 #include <vlc_demux.h>
 #include <vlc_input.h>
 #include <vlc_vout.h>
+#include <vlc_aout.h>
+#include <vlc_charset.h>
 
 #include "libvlc.h"
 
@@ -1250,3 +1252,210 @@ int libvlc_media_player_can_pause( libvlc_media_player_t *p_mi,
 
     return val.b_bool;
 }
+
+/*****************************************************************************
+ * Equalizer: set and get libvlc_audio_equalizer_t
+ *****************************************************************************/
+static aout_instance_t *GetAOut( libvlc_instance_t *p_instance,
+                                 libvlc_exception_t *p_exception )
+{
+    if( !p_instance )
+        return NULL;
+
+    aout_instance_t * p_aout = NULL;
+
+    p_aout = vlc_object_find( p_instance->p_libvlc_int, VLC_OBJECT_AOUT, FIND_CHILD );
+    if( !p_aout )
+    {
+        libvlc_exception_raise( p_exception, "No active audio output" );
+        return NULL;
+    }
+
+    return p_aout;
+}
+
+static inline int set_eqz( vlc_object_t * p_aout,
+                           libvlc_audio_equalizer_t * p_eqz,
+                           libvlc_exception_t * p_e )
+{
+
+    if( ( p_eqz == NULL ) || ( p_aout == NULL ) )
+    {
+        libvlc_exception_raise(p_e, "Invalid argument");
+        return VLC_EGENERIC;
+    }
+
+    bool b_neg = false;
+    char *psz_newbands = NULL;
+
+    for( int i = 0; i < p_eqz->i_band ; i++ )
+    {
+        lldiv_t d;
+        float f_tmp;
+        char *psz;
+
+        /* this is to avoid problem with negative value for lldiv */
+        if( p_eqz->f_band[i] < 0 )
+        {
+            f_tmp = - p_eqz->f_band[i];
+            b_neg = true;
+        }
+        else
+        {
+            f_tmp = p_eqz->f_band[i];
+            b_neg = false;
+        }
+
+        d = lldiv( f_tmp * 10000000, 10000000 );
+
+        if( b_neg == true)
+        {
+            d.quot = -d.quot;
+        }
+
+        if( asprintf( &psz, "%s %lld.%07llu",
+                      psz_newbands ? psz_newbands : "",
+                      d.quot, d.rem ) == -1 )
+        {
+            free( psz_newbands );
+            return VLC_EGENERIC;
+        }
+        psz_newbands = psz;
+    }
+
+    var_SetString( p_aout, "equalizer-bands", psz_newbands );
+    var_SetFloat( p_aout, "equalizer-preamp", p_eqz->f_preamp );
+
+    return VLC_SUCCESS;
+}
+
+static inline int get_eqz( vlc_object_t * p_aout,
+                           libvlc_audio_equalizer_t * p_eqz,
+                           libvlc_exception_t *p_e )
+{
+    vlc_value_t val, val_preamp;
+    int i;
+
+    if( ( p_eqz == NULL ) || ( p_aout == NULL ) )
+    {
+        libvlc_exception_raise(p_e, "Invalid argument");
+        return VLC_EGENERIC;
+    }
+
+    /* get the current band values */
+    if( var_Get( p_aout, "equalizer-bands", &val ) < 0 )
+    {
+        /* if unable to get it return error */
+        libvlc_exception_raise(p_e,"unable to get equalizer bands values");
+        return VLC_EGENERIC;
+    }
+    char *psz_bands = val.psz_string;
+    char *psz_next;
+    char *p = psz_bands;
+
+    for( i = 0 ; i < EQZ_BANDS_MAX ; i++ )
+    {
+        float f_val;
+
+        if( *psz_bands == '\0' )
+            break;
+
+        /* get values from string */
+#ifdef HAVE_STRTOF
+        f_val = us_strtof( p, &psz_next );
+#else
+        f_val = (float)us_strtod( p, &psz_next );
+#endif
+
+        p_eqz->f_band[i] = f_val;
+
+        if( psz_next == p )
+            break; /* no conversion */
+
+        if( *psz_next == '\0' )
+            break; /* end of line */
+        p = &psz_next[1];
+    }
+
+    /* get the preamp value */
+    if( var_Get( p_aout, "equalizer-preamp", &val_preamp ) < 0 )
+    {
+        libvlc_exception_raise(p_e, "unable to get equalizer preamp value");
+        return VLC_EGENERIC;
+    }
+    p_eqz->f_preamp = val_preamp.f_float;
+    p_eqz->i_band = i+1;
+
+    return VLC_SUCCESS;
+}
+
+/*****************************************************************************
+ * libvlc_media_player_get_equalizer: get current equalizer settings and
+ *                                    save it into a libvlc_audio_equalizer_t
+ *****************************************************************************/
+void libvlc_media_player_get_equalizer( libvlc_instance_t * p_instance,
+                                        libvlc_audio_equalizer_t * p_eqz,
+                                        libvlc_exception_t *p_e )
+{
+    aout_instance_t *p_aout = GetAOut( p_instance, p_e );
+
+    if ( !p_aout )
+        return;
+
+    /* get equalizer status (enable or disable) */
+    vlc_value_t val;
+    if( var_Get( VLC_OBJECT( p_aout ), "audio-filter", &val) < 0 )
+    {
+        /* if unable to get it return error */
+        libvlc_exception_raise(p_e,"unable to get equalizer status");
+        p_eqz = NULL;
+    }
+    const char *psz_parser = strstr( val.psz_string, "equalizer" );
+    if( !psz_parser )
+        p_eqz = NULL;
+
+    /* if NULL don't get the current config */
+    if( p_eqz == NULL )
+    {
+        vlc_object_release( p_aout );
+        return;
+    }
+
+    /* get current config */
+    if( get_eqz( p_aout, p_eqz, p_e ) == VLC_EGENERIC )
+        p_eqz = NULL;
+
+    vlc_object_release( p_aout );
+}
+
+/*****************************************************************************
+ * libvlc_media_player_set_equalizer: set to equalizer values contained in
+ *                                    libvlc_audio_equalizer_t
+ *****************************************************************************/
+void libvlc_media_player_set_equalizer( libvlc_instance_t * p_instance,
+                                        libvlc_audio_equalizer_t * p_eqz,
+                                        libvlc_exception_t * p_e )
+{
+    aout_instance_t *p_aout = GetAOut( p_instance, p_e );
+
+    if ( !p_aout )
+        return;
+
+    /* enable or disable equalizer */
+    if( p_eqz != NULL )
+    {
+        aout_EnableFilter( VLC_OBJECT( p_aout ), "equalizer", true );
+    }
+    else
+    {
+        aout_EnableFilter( VLC_OBJECT( p_aout ), "equalizer", false );
+        return;
+    }
+
+    /* set the equalizer configuration */
+    if( set_eqz(p_aout, p_eqz, p_e) == VLC_EGENERIC )
+        libvlc_exception_raise( p_e, "Unable to set equalizer configuration" );
+
+    vlc_object_release( p_aout );
+    return;
+}
diff --git a/src/libvlc.sym b/src/libvlc.sym
index 1583c31..f1b1e93 100644
--- a/src/libvlc.sym
+++ b/src/libvlc.sym
@@ -1,4 +1,14 @@
 libvlc_add_intf
+libvlc_audio_equalizer_get_amp_at_index
+libvlc_audio_equalizer_get_band_count
+libvlc_audio_equalizer_get_preamp
+libvlc_audio_equalizer_get_preset_name
+libvlc_audio_equalizer_new
+libvlc_audio_equalizer_presets_get_count
+libvlc_audio_equalizer_release
+libvlc_audio_equalizer_set_amp_at_index
+libvlc_audio_equalizer_set_preamp
+libvlc_audio_equalizer_set_preset
 libvlc_audio_output_device_count
 libvlc_audio_output_device_id
 libvlc_audio_output_device_longname
@@ -125,6 +135,7 @@ libvlc_media_player_get_chapter
 libvlc_media_player_get_chapter_count
 libvlc_media_player_get_chapter_count_for_title
 libvlc_media_player_get_drawable
+libvlc_media_player_get_equalizer
 libvlc_media_player_get_fps
 libvlc_media_player_get_hwnd
 libvlc_media_player_get_length
@@ -151,6 +162,7 @@ libvlc_media_player_retain
 libvlc_media_player_set_agl
 libvlc_media_player_set_chapter
 libvlc_media_player_set_drawable
+libvlc_media_player_set_equalizer
 libvlc_media_player_set_hwnd
 libvlc_media_player_set_media
 libvlc_media_player_set_nsobject
-- 
1.5.4.3

